@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2003 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,16 +15,14 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif /* HAVE_CONFIG_H */
+#include "prologue.h"
 
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
  
-#include "contract.h"
-#include "ctb_definitions.h"
+#include "ctb.h"
+#include "ctb_internal.h"
 #include "brl.h"
 
 static const ContractionTableHeader *table;	/*translation table */
@@ -63,134 +61,186 @@ setAfter (int length) {
 static int
 selectRule (int length) { /*check for valid contractions */
   int ruleOffset;
+  int maximumLength;
 
   if (length < 1) return 0;
   if (length == 1) {
-    ruleOffset = table->characters[*src].rules;
+    ruleOffset = table->characters[CTL(*src)].rules;
+    maximumLength = 1;
   } else {
     BYTE bytes[2];
     bytes[0] = CTL(src[0]);
     bytes[1] = CTL(src[1]);
     ruleOffset = table->rules[hash(bytes)];
+    maximumLength = 0;
   }
+
   while (ruleOffset) {
     currentRule = CTR(table, ruleOffset);
     currentOpcode = currentRule->opcode;
     currentFindLength = currentRule->findlen;
+
     if ((length == 1) ||
-        ((currentFindLength <= length) && compareBytes(src, currentRule->findrep, currentFindLength))) {
-      /* check this rule */
+        ((currentFindLength <= length) &&
+         compareBytes(src, currentRule->findrep, currentFindLength))) {
       setAfter(currentFindLength);
-      switch (currentOpcode) { /*check validity of this contraction */
-        case CTO_Always:
-        case CTO_Repeated:
-        case CTO_Replace:
-        case CTO_Literal:
-          return 1;
-        case CTO_LargeSign:
-          if (!CTC(before, CTC_Space) || !CTC(after, CTC_Space))
-            currentOpcode = CTO_Always;
-          return 1;
-        case CTO_WholeWord:
-        case CTO_Contraction:
-          if (CTC(before, CTC_Space|CTC_Punctuation) &&
-              CTC(after, CTC_Space|CTC_Punctuation))
-            return 1;
-          break;
-        case CTO_LowWord:
-          if (CTC(before, CTC_Space) && CTC(after, CTC_Space) &&
-              (previousOpcode != CTO_JoinableWord))
-            return 1;
-          break;
-        case CTO_JoinableWord:
-          if (CTC(before, CTC_Space|CTC_Punctuation) &&
-              CTC(after, CTC_Space) &&
-              (dest + currentRule->replen < destmax)) {
-            const BYTE *ptr = src + currentFindLength + 1;
-            while (ptr < srcmax) {
-              if (!CTC(*ptr, CTC_Space)) {
-                if (CTC(*ptr, CTC_Letter)) return 1;
-                break;
-              }
-              ptr++;
-            }
+
+      if (!maximumLength) {
+#define STATE(c) (CTC((c), CTC_UpperCase)? 1: CTC((c), CTC_LowerCase)? 0: -1)
+
+        int state = STATE(before);
+        int i;
+        maximumLength = currentFindLength;
+
+        for (i=0; i<currentFindLength; ++i) {
+          unsigned char byte = src[i];
+          int next = STATE(byte);
+
+          if ((i > 0) &&
+              (((state == 0) && (next == 1)) ||
+               ((state == 2) && (next == 0)))) {
+            maximumLength = i;
+            break;
           }
-          break;
-        case CTO_SuffixableWord:
-          if (CTC(before, CTC_Space|CTC_Punctuation) &&
-              CTC(after, CTC_Space|CTC_Letter|CTC_Punctuation))
-            return 1;
-          break;
-        case CTO_BegWord:
-          if (CTC(before, CTC_Space|CTC_Punctuation) &&
-              CTC(after, CTC_Letter))
-            return 1;
-          break;
-        case CTO_MidWord:
-          if (CTC(before, CTC_Letter) && CTC(after, CTC_Letter))
-            return 1;
-          break;
-        case CTO_MidEndWord:
-          if (CTC(before, CTC_Letter) &&
-              CTC(after, CTC_Letter|CTC_Space|CTC_Punctuation))
-            return 1;
-          break;
-        case CTO_EndWord:
-          if (CTC(before, CTC_Letter) &&
-              CTC(after, CTC_Space|CTC_Punctuation))
-            return 1;
-          break;
-        case CTO_BegNum:
-          if (CTC(before, CTC_Space|CTC_Punctuation) &&
-              CTC(after, CTC_Digit))
-            return 1;
-          break;
-        case CTO_MidNum:
-          if (CTC(before, CTC_Digit) && CTC(after, CTC_Digit))
-            return 1;
-          break;
-        case CTO_EndNum:
-          if (CTC(before, CTC_Digit) &&
-              CTC(after, CTC_Space|CTC_Punctuation))
-            return 1;
-          break;
-        {
-          int isPre;
-        case CTO_PrePunc:
-          isPre = 1;
-          goto doPunc;
-        case CTO_PostPunc:
-          isPre = 0;
-        doPunc:
-          if (CTC(*src, CTC_Punctuation)) {
-            const BYTE *pre = src;
-            const BYTE *post = src + currentFindLength;
-            while (--pre >= srcmin)
-              if (!CTC(*pre, CTC_Punctuation))
-                break;
-            while (post < srcmax) {
-              if (!CTC(*post, CTC_Punctuation))
-                break;
-              post++;
-            }
-            if (isPre) {
-              if (((pre < srcmin) || CTC(*pre, CTC_Space)) &&
-                  ((post < srcmax) && !CTC(*post, CTC_Space)))
-                return 1;
-            } else {
-              if (((pre >= srcmin) && !CTC(*pre, CTC_Space)) &&
-                  ((post == srcmax) || CTC(*post, CTC_Space)))
-                return 1;
-            }
+
+          if ((state > 0) && (next == 1)) {
+            state = 2;
+          } else {
+            state = next;
           }
-          break;
         }
-        default:
-          break;
+
+#undef STATE
+      }
+
+      if ((currentFindLength <= maximumLength) &&
+          (!currentRule->after || CTC(before, currentRule->after)) &&
+          (!currentRule->before || CTC(after, currentRule->before))) {
+        switch (currentOpcode) {
+          case CTO_Always:
+          case CTO_Repeated:
+          case CTO_Replace:
+          case CTO_Literal:
+            return 1;
+          case CTO_LargeSign:
+          case CTO_LastLargeSign:
+            if (!CTC(before, CTC_Space) || !CTC(after, CTC_Space))
+              currentOpcode = CTO_Always;
+            return 1;
+          case CTO_WholeWord:
+          case CTO_Contraction:
+            if (CTC(before, CTC_Space|CTC_Punctuation) &&
+                CTC(after, CTC_Space|CTC_Punctuation))
+              return 1;
+            break;
+          case CTO_LowWord:
+            if (CTC(before, CTC_Space) && CTC(after, CTC_Space) &&
+                (previousOpcode != CTO_JoinableWord) &&
+                ((dest == destmin) || !dest[-1]))
+              return 1;
+            break;
+          case CTO_JoinableWord:
+            if (CTC(before, CTC_Space|CTC_Punctuation) &&
+                CTC(after, CTC_Space) &&
+                (dest + currentRule->replen < destmax)) {
+              const BYTE *ptr = src + currentFindLength + 1;
+              while (ptr < srcmax) {
+                if (!CTC(*ptr, CTC_Space)) {
+                  if (CTC(*ptr, CTC_Letter)) return 1;
+                  break;
+                }
+                ptr++;
+              }
+            }
+            break;
+          case CTO_SuffixableWord:
+            if (CTC(before, CTC_Space|CTC_Punctuation) &&
+                CTC(after, CTC_Space|CTC_Letter|CTC_Punctuation))
+              return 1;
+            break;
+          case CTO_PrefixableWord:
+            if (CTC(before, CTC_Space|CTC_Letter|CTC_Punctuation) &&
+                CTC(after, CTC_Space|CTC_Punctuation))
+              return 1;
+            break;
+          case CTO_BegWord:
+            if (CTC(before, CTC_Space|CTC_Punctuation) &&
+                CTC(after, CTC_Letter))
+              return 1;
+            break;
+          case CTO_BegMidWord:
+            if (CTC(before, CTC_Letter|CTC_Space|CTC_Punctuation) &&
+                CTC(after, CTC_Letter))
+              return 1;
+            break;
+          case CTO_MidWord:
+            if (CTC(before, CTC_Letter) && CTC(after, CTC_Letter))
+              return 1;
+            break;
+          case CTO_MidEndWord:
+            if (CTC(before, CTC_Letter) &&
+                CTC(after, CTC_Letter|CTC_Space|CTC_Punctuation))
+              return 1;
+            break;
+          case CTO_EndWord:
+            if (CTC(before, CTC_Letter) &&
+                CTC(after, CTC_Space|CTC_Punctuation))
+              return 1;
+            break;
+          case CTO_BegNum:
+            if (CTC(before, CTC_Space|CTC_Punctuation) &&
+                CTC(after, CTC_Digit))
+              return 1;
+            break;
+          case CTO_MidNum:
+            if (CTC(before, CTC_Digit) && CTC(after, CTC_Digit))
+              return 1;
+            break;
+          case CTO_EndNum:
+            if (CTC(before, CTC_Digit) &&
+                CTC(after, CTC_Space|CTC_Punctuation))
+              return 1;
+            break;
+          {
+            int isPre;
+          case CTO_PrePunc:
+            isPre = 1;
+            goto doPunc;
+          case CTO_PostPunc:
+            isPre = 0;
+          doPunc:
+            if (CTC(*src, CTC_Punctuation)) {
+              const BYTE *pre = src;
+              const BYTE *post = src + currentFindLength;
+              while (--pre >= srcmin)
+                if (!CTC(*pre, CTC_Punctuation))
+                  break;
+              while (post < srcmax) {
+                if (!CTC(*post, CTC_Punctuation))
+                  break;
+                post++;
+              }
+              if (isPre) {
+                if (((pre < srcmin) || CTC(*pre, CTC_Space)) &&
+                    ((post < srcmax) && !CTC(*post, CTC_Space)))
+                  return 1;
+              } else {
+                if (((pre >= srcmin) && !CTC(*pre, CTC_Space)) &&
+                    ((post == srcmax) || CTC(*post, CTC_Space)))
+                  return 1;
+              }
+            }
+            break;
+          }
+          default:
+            break;
+        }
       }
     }				/*Done with checking this rule */
+
     ruleOffset = currentRule->next;
   }
+
   return 0;
 }
 
@@ -247,8 +297,7 @@ contractText (void *contractionTable,
               int *offsetsMap, const int cursorOffset) { /*begin translation */
   const BYTE *srcword = NULL;
   BYTE *destword = NULL; /* last word transla1ted */
-  int computerBraille = 0;
-  const BYTE *activeBegin, *activeEnd;
+  const BYTE *computerBraille = NULL;
 
   if (!(table = (ContractionTableHeader *) contractionTable)) return 0;
   srcmax = (srcmin = src = inputBuffer) + *inputLength;
@@ -256,41 +305,36 @@ contractText (void *contractionTable,
   offsets = offsetsMap;
   previousOpcode = CTO_None;
 
-  activeBegin = activeEnd = NULL;
-  if (table->locale) {
-    if (cursorOffset >= 0 && cursorOffset < *inputLength &&
-        !CTC(inputBuffer[cursorOffset], CTC_Space)) {
-      activeBegin = activeEnd = srcmin + cursorOffset;
-      while (--activeBegin >= srcmin && !CTC(*activeBegin, CTC_Space));
-      while (++activeEnd < srcmax && !CTC(*activeEnd, CTC_Space));
-    }
-  }
-
   while (src < srcmax) { /*the main translation loop */
     setOffset();
     setBefore();
     if (computerBraille)
-      if (CTC(*src, CTC_Space))
-        computerBraille = 0;
-    if (computerBraille || (src > activeBegin && src < activeEnd)) {
+      if (src >= computerBraille)
+        if (CTC(*src, CTC_Space) || CTC(src[-1], CTC_Space))
+          computerBraille = NULL;
+    if (computerBraille) {
       setAfter(1);
       currentOpcode = CTO_Literal;
       if (!putBytes(&textTable[*src], 1)) break;
       src++;
     } else if (selectRule(srcmax-src) || selectRule(1)) {
-      if (table->numberSign && previousOpcode != CTO_MidNum &&
-          !CTC(before, CTC_Digit) && CTC(*src, CTC_Digit)) {
+      if (table->locale &&
+          (cursorOffset >= (src - srcmin)) &&
+          (cursorOffset < (src - srcmin + currentFindLength))) {
+        currentOpcode = CTO_Literal;
+      } else if (table->numberSign && previousOpcode != CTO_MidNum &&
+                 !CTC(before, CTC_Digit) && CTC(*src, CTC_Digit)) {
         if (!putSequence(table->numberSign)) break;
       } else if (table->englishLetterSign && CTC(*src, CTC_Letter)) {
         if ((currentOpcode == CTO_Contraction) ||
             (currentOpcode != CTO_EndNum && CTC(before, CTC_Digit)) ||
             (currentOpcode == CTO_Always && currentFindLength == 1 && CTC(before, CTC_Space) &&
              (src + 1 == srcmax || CTC(src[1], CTC_Space) ||
-              (CTC(src[1], CTC_Punctuation) &&
-              !(src + 2 < srcmax && src[1] == '.' && CTC(src[2], CTC_Letter)))))) {
+              (CTC(src[1], CTC_Punctuation) && (src[1] != '.'))))) {
           if (!putSequence(table->englishLetterSign)) break;
         }
       }
+
       if (CTC(*src, CTC_UpperCase)) {
         if (!CTC(before, CTC_UpperCase)) {
           if (table->beginCapitalSign &&
@@ -310,6 +354,7 @@ contractText (void *contractionTable,
       /* pre processing */
       switch (currentOpcode) {
         case CTO_LargeSign:
+        case CTO_LastLargeSign:
           if (previousOpcode == CTO_LargeSign) {
             int srcoff = src - srcmin;
             int destlen;
@@ -337,6 +382,8 @@ contractText (void *contractionTable,
           break;
         case CTO_Literal: {
           const BYTE *srcorig = src;
+          computerBraille = src + currentFindLength;
+          if (CTC(*src, CTC_Space)) continue;
           if (destword) {
             src = srcword;
             dest = destword;
@@ -345,7 +392,6 @@ contractText (void *contractionTable,
             dest = destmin;
           }
           while (srcorig > src) offsets[--srcorig - srcmin] = -1;
-          computerBraille = 1;
           continue;
         }
         default:
