@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,12 +15,10 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#ifndef _MISC_H
-#define _MISC_H
+#ifndef BRLTTY_INCLUDED_MISC
+#define BRLTTY_INCLUDED_MISC
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif /* HAVE_CONFIG_H */
+#include "prologue.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -65,6 +63,26 @@ extern "C" {
 #define _STRINGIFY(a) #a
 #define STRINGIFY(a) _STRINGIFY(a)
 
+#ifdef WINDOWS
+#define getSystemError() GetLastError()
+#define getSocketError() WSAGetLastError()
+
+#ifdef __CYGWIN__
+#include <sys/cygwin.h>
+#define setErrno(error) errno = cygwin_internal(CW_GET_ERRNO_FROM_WINERROR, (error))
+#else /* __CYGWIN__ */
+#define setErrno(error) errno = EIO
+#endif /* __CYGWIN__ */
+#else /* WINDOWS */
+#define getSystemError() errno
+#define getSocketError() errno
+
+#define setErrno(error)
+#endif /* WINDOWS */
+
+#define setSystemErrno() setErrno(getSystemError())
+#define setSocketErrno() setErrno(getSocketError())
+
 extern char **splitString (const char *string, char delimiter, int *count);
 extern void deallocateStrings (char **array);
 
@@ -79,6 +97,11 @@ extern int processLines (FILE *file, /* The input file. */
 		         void *data); /* A pointer to caller-specific data. */
 extern int readLine (FILE *file, char **buffer, size_t *size);
 
+#ifdef __MINGW32__
+extern void gettimeofday (struct timeval *tvp, void *tzp);
+extern void usleep (int usec);
+#endif /* __MINGW32__ */
+
 extern void approximateDelay (int milliseconds);		/* sleep for `msec' milliseconds */
 extern void accurateDelay (int milliseconds);
 extern long int millisecondsBetween (const struct timeval *from, const struct timeval *to);
@@ -108,8 +131,14 @@ extern void LogPrint
 #endif /* HAVE_ATTRIBUTE_FORMAT_PRINTF */
        ;
 extern void LogError (const char *action);
+#ifdef WINDOWS
+extern void LogWindowsCodeError (DWORD code, const char *action);
+extern void LogWindowsError (const char *action);
+extern void LogWindowsSocketError (const char *action);
+#endif /* WINDOWS */
 extern void LogBytes (const char *description, const unsigned char *data, unsigned int length);
 extern int setLogLevel (int level);
+extern const char *setPrintPrefix (const char *prefix);
 extern int setPrintLevel (int level);
 extern int setPrintOff (void);
 extern int loggedProblemCount;
@@ -121,6 +150,8 @@ extern int ringBell (void);
 extern void *mallocWrapper (size_t size);
 extern void *reallocWrapper (void *address, size_t size);
 extern char *strdupWrapper (const char *string);
+
+extern char *getWorkingDirectory (void);
 extern char *makePath (const char *directory, const char *file);
 extern int makeDirectory (const char *path);
 
@@ -128,8 +159,11 @@ extern char *getDevicePath (const char *path);
 extern int isQualifiedDevice (const char **path, const char *qualifier);
 extern void unsupportedDevice (const char *path);
 
+extern int rescaleInteger (int value, int from, int to);
+
 extern int isInteger (int *value, const char *word);
 extern int isFloat (float *value, const char *word);
+
 extern int validateInteger (int *value, const char *description, const char *word, const int *minimum, const int *maximum);
 extern int validateFloat (float *value, const char *description, const char *word, const float *minimum, const float *maximum);
 extern int validateChoice (unsigned int *value, const char *description, const char *word, const char *const *choices);
@@ -141,4 +175,4 @@ extern int validateYesNo (unsigned int *value, const char *description, const ch
 }
 #endif /* __cplusplus */
 
-#endif /* _MISC_H */
+#endif /* BRLTTY_INCLUDED_MISC */
