@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,8 +15,8 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#ifndef _SCR_FROZEN_H
-#define _SCR_FROZEN_H
+#ifndef BRLTTY_INCLUDED_SCR_FROZEN
+#define BRLTTY_INCLUDED_SCR_FROZEN
 
 #ifdef __cplusplus
 extern "C" {
@@ -36,4 +36,4 @@ extern void initializeFrozenScreen (FrozenScreen *);
 }
 #endif /* __cplusplus */
 
-#endif /* _SCR_FROZEN_H */
+#endif /* BRLTTY_INCLUDED_SCR_FROZEN */
