@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2003 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,8 +15,8 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#ifndef _SCR_HELP_H
-#define _SCR_HELP_H
+#ifndef BRLTTY_INCLUDED_SCR_HELP
+#define BRLTTY_INCLUDED_SCR_HELP
 
 #ifdef __cplusplus
 extern "C" {
@@ -40,4 +40,4 @@ extern void initializeHelpScreen (HelpScreen *);
 }
 #endif /* __cplusplus */
 
-#endif /* _SCR_HELP_H */
+#endif /* BRLTTY_INCLUDED_SCR_HELP */
