@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,8 +15,8 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#ifndef _SCR_MAIN_H
-#define _SCR_MAIN_H
+#ifndef BRLTTY_INCLUDED_SCR_MAIN
+#define BRLTTY_INCLUDED_SCR_MAIN
 
 #ifdef __cplusplus
 extern "C" {
@@ -26,18 +26,17 @@ extern "C" {
 
 typedef struct {
   BaseScreen base;
-  const char *const * (*parameters) (void);
   int (*prepare) (char **parameters);
   int (*open) (void);
   int (*setup) (void);
   void (*close) (void);
+  int (*uservt) (int number);
 } MainScreen;
 
 extern void initializeMainScreen (MainScreen *);
-extern void initializeLiveScreen (MainScreen *);
 
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
 
-#endif /* _SCR_MAIN_H */
+#endif /* BRLTTY_INCLUDED_SCR_MAIN */
