@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2003 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,9 +15,7 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif /* HAVE_CONFIG_H */
+#include "prologue.h"
 
 #include <stdlib.h>
 #include <unistd.h>
@@ -30,29 +28,27 @@
 
 const void *
 loadDriver (
-  const char *driver,
-  const char *driverDirectory, const char *driverSymbol,
-  const char *driverType, char driverCharacter,
-  const DriverEntry *driverTable,
-  const void *nullAddress, const char *nullIdentifier
+  const char *driverCode, void **driverObject,
+  const char *driverDirectory, const DriverEntry *driverTable,
+  const char *driverType, char driverCharacter, const char *driverSymbol,
+  const void *nullAddress, const char *nullCode
 ) {
   const void *driverAddress = NULL;
+  *driverObject = NULL;
 
-  if (!driver) {
+  if (!driverCode || !*driverCode) {
     if (driverTable)
       if (driverTable->address)
         return driverTable->address;
     return nullAddress;
   }
 
-  if (strcmp(driver, nullIdentifier) == 0) {
-    return nullAddress;
-  }
+  if (strcmp(driverCode, nullCode) == 0) return nullAddress;
 
   if (driverTable) {
     const DriverEntry *driverEntry = driverTable;
     while (driverEntry->address) {
-      if (strcmp(driver, *driverEntry->identifier) == 0) {
+      if (strcmp(driverCode, *driverEntry->code) == 0) {
         return driverEntry->address;
       }
       ++driverEntry;
@@ -65,26 +61,28 @@ loadDriver (
     const char *symbolName;
 
     {
-      int length = strlen(LIBRARY_NAME) + strlen(driver) + strlen(LIBRARY_EXTENSION) + 3;
+      int length = strlen(MODULE_NAME) + strlen(driverCode) + strlen(MODULE_EXTENSION) + 3;
       char *name = mallocWrapper(length);
       snprintf(name, length, "%s%c%s.%s",
-               LIBRARY_NAME, driverCharacter, driver, LIBRARY_EXTENSION);
+               MODULE_NAME, driverCharacter, driverCode, MODULE_EXTENSION);
       libraryName = name;
     }
     libraryPath = makePath(driverDirectory, libraryName);
 
     {
-      int length = strlen(driverSymbol) + strlen(driver) + 2;
+      int length = strlen(driverSymbol) + strlen(driverCode) + 2;
       char *name = mallocWrapper(length);
       snprintf(name, length, "%s_%s",
-               driverSymbol, driver);
+               driverSymbol, driverCode);
       symbolName = name;
     }
 
     {
       void *libraryHandle = loadSharedObject(libraryPath);
       if (libraryHandle) {
-        if (!findSharedSymbol(libraryHandle, symbolName, &driverAddress)) {
+        if (findSharedSymbol(libraryHandle, symbolName, &driverAddress)) {
+          *driverObject = libraryHandle;
+        } else {
           LogPrint(LOG_ERR, "Cannot find %s driver symbol: %s", driverType, symbolName);
           unloadSharedObject(libraryHandle);
           driverAddress = NULL;
