@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2003 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,8 +15,8 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#ifndef _SYSMISC_H
-#define _SYSMISC_H
+#ifndef BRLTTY_INCLUDED_SYSMISC
+#define BRLTTY_INCLUDED_SYSMISC
 
 #ifdef __cplusplus
 extern "C" {
@@ -24,20 +24,18 @@ extern "C" {
 
 typedef struct {
   const void *address;
-  const char *const *identifier;
+  const char *const *code;
 } DriverEntry;
 
-extern const void *
-loadDriver (
-  const char *driver,
-  const char *driverDirectory, const char *driverSymbol,
-  const char *driverType, char driverCharacter,
-  const DriverEntry *driverTable,
-  const void *nullAddress, const char *nullIdentifier
+extern const void *loadDriver (
+  const char *driverCode, void **driverObject,
+  const char *driverDirectory, const DriverEntry *driverTable,
+  const char *driverType, char driverCharacter, const char *driverSymbol,
+  const void *nullAddress, const char *nullCode
 );
 
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
 
-#endif /* _SYSMISC_H */
+#endif /* BRLTTY_INCLUDED_SYSMISC */
